#include <ddn/ddn_gt0.h>
#include <ddn/ddn_pt0.h>
#include <ddn/ddn.h>

#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <getopt.h>
#include <stdio.h>

#define OPT_EXT 0x7f

const char *prog;

static void
usage(void)
{
	fprintf(stderr, "usage: %s\n\n", prog);
	fprintf(stderr, "  -h, --help                   Show this help message.\n");
	fprintf(stderr, "  -i, --iterations <count>     Run for <count> iterations.\n");
	fprintf(stderr, "  -V, --version                Show version information.\n");
}

static void
version(void)
{
	fprintf(stderr, "ddnsim v0\n");
}

size_t opt_iterations = 20;

static const struct option opt_long[] =
{
	{ "help", 0, NULL, 'h' },
	{ "iterations", 1, NULL, 'i' },
	{ "version", 0, NULL, 'V' },
	{ NULL, 0, NULL, 0 },
};

static const char opt_short[] = ":hi:V";

int
main(int argc, char **argv)
{
	struct genotype *gt;
	struct phenotype *pt;
	struct ddn *ddn;

	prog = basename(argv[0]);

	while (1)
	{
		int opt;

		opt = getopt_long(argc, argv, opt_short, opt_long, NULL);
		if (opt == -1)
			break;
		switch (opt)
		{
		case 'h':
			usage();
			return 0;
		case 'i':
			opt_iterations = atoi(optarg);
			break;
		case 'V':
			version();
			return 0;
		case '?':
			fprintf(stderr, "%s: error: unknown option\n", prog);
			return 1;
		case ':':
			fprintf(stderr, "%s: error: missing argument\n", prog);
			return 2;
		default:
			fprintf(stderr, "%s: error: weird return value '%c'\n", prog, opt);
			return 3;
		}
	}

	gt = ddn_gt0_fscan(stdin);
	if (!gt)
	{
		fprintf(stderr, "%s: error: failed to parse genotype\n", prog);
		return 6;
	}
	//genotype_fprint(gt, stdout);

	pt = ddn_pt0_create(structof(gt, base, const struct ddn_gt0));
	if (!pt)
	{
		fprintf(stderr, "%s: error: failed to develop phenotype\n", prog);
		return 7;
	}

	ddn = ddn_pt0_ddn(pt);
	while (!feof(stdin) && !ferror(stdin))
	{
		int tmp;

		tmp = ddn_state_fscan(ddn, stdin);
		if (tmp < 0)
		{
			fprintf(stderr, "%s: error: failed to parse ddn initial state\n", prog);
			return 13;
		}
		else if (tmp > 0)
			break;

		ddn_state_fprint(ddn, stdout);
		fputc('\n', stdout);

		for (size_t i = 1; i < opt_iterations; ++i)
		{
			ddn_step(ddn);
			ddn_state_fprint(ddn, stdout);
			fputc('\n', stdout);
		}
	}

	if (ferror(stdin))
	{
		fprintf(stderr, "%s: error: %s\n", prog, strerror(errno));
		return 10;
	}

	if (fflush(stdout))
	{
		fprintf(stderr, "%s: error: %s\n", prog, strerror(errno));
		return 11;
	}

	if (ferror(stdout))
	{
		fprintf(stderr, "%s: error: %s\n", prog, strerror(errno));
		return 12;
	}
	return 0;
}
