#!/usr/bin/env bash

adults=64
adult_sel_mech=stochastic
adult_sel_proto=full-gen-rep
crossover_prob=0.03
children=64
elites=1
generation_print=128
generations=10240
log=expr1.$$.log
#log=expr1.1928.log
mutation_prob=0.18
nodes=8
dft_winsize=512
dft_winfunc=rect
sim_statecount=2048
parent_sel_mech=sigma-scale
crossover_points=1
poplog=expr1-pop.$$.log

cat > "$log.states" << EOF
00111100
EOF
#00000001
#00000000
#11111111
#11000100
#00010101
#10001000
#00111100
#11000011

export MALLOC_CHECK_=3

../ddnevo \
	--adults "$adults" \
	--adult-sel-mech "$adult_sel_mech" \
	--adult-sel-proto "$adult_sel_proto" \
	--crossover-prob "$crossover_prob" \
	--children "$children" \
	--elites "$elites" \
	--generation-print "$generation_print" \
	--generations "$generations" \
	--log "$log" \
	--mutation-prob "$mutation_prob" \
	--nodes "$nodes" \
	--option dft-winsize="$dft_winsize" \
	--option dft-winfunc="$dft_winfunc" \
	--option sim-statecount="$sim_statecount" \
	--parent-sel-mech="$parent_sel_mech" \
	--crossover-points "$crossover_points" \
	--population-log "$poplog" \
	| tee "$log.out"

i=0
for state in `cat "$log.states"` ; do
	file="$log.$i"
	tail -n 1 "$log"|awk '{ print $11, $12 }' > "$file.in"
	echo "$state" >> "$file.in"

	../ddnsim \
		--iterations "$sim_statecount" \
		< "$file.in" > "$file.sim"
	../ddndft \
		--nodes "$nodes" \
		--winsize "$dft_winsize" \
		--winfunc "$dft_winfunc" \
		< "$file.sim" > "$file.dft"
	../dftplot.sh < "$file.dft" > "$file.dft.pdf"

	../ddncov \
		--nodes "$nodes" \
		< "$file.sim" > "$file.cov"
	../covplot.sh < "$file.cov" > "$file.cov.pdf"

	i=`expr $i + 1`
done

tail -n 1 "$log"|awk '{ print $11, $12 }' | ../ddn2dot > "$log.dot"
dot -Tpdf -o"$log.dot.pdf" "$log.dot"
../fitplot.sh "$log" > "$log.fit.pdf"

