#ifndef DBG_H_
#define DBG_H_ 1

#define DBG_TRACE_FATAL		0
#define DBG_TRACE_ERROR		1
#define DBG_TRACE_WARN		2
#define DBG_TRACE_NOTICE	3
#define DBG_TRACE_INFO		4
#define DBG_TRACE_DEBUG		5

#define DBG_TRACE_LEVEL_DEFAULT DBG_TRACE_DEBUG

void dbg_printf(int level, const char *file, int line, const char *fmt, ...);

extern int dbg_trace_level;

#ifdef DEBUG

#include <mcheck.h>
#define dbg_mtrace() mtrace()
#define dbg_trace(level, fmt, args...) \
	do \
	{ \
		if (level <= dbg_trace_level) \
			dbg_printf(level, __FILE__, __LINE__, fmt, ## args); \
	} while (0)

#define dbg_trace_count(level, count, fmt, args...) \
	do \
	{ \
		static int __count = 0; \
		if (__count < (count)) \
		{ \
			++__count; \
			dbg_trace(level, fmt, ## args); \
		} \
	} while (0)

#define _dbg_assert(cond, fatal) \
	do \
	{ \
		if (!(cond)) \
		{ \
			dbg_warn("assert '%s' failed\n", #cond); \
			if (fatal) \
				abort(); \
		} \
	} while (0)
#define dbg_assert(cond) _dbg_assert(cond, 0)
#define dbg_assert_fatal(cond) _dbg_assert(cond, 1)
#ifdef assert
#undef assert
#endif
#define assert(cond) dbg_assert(cond)

#else /* !DEBUG */

#define dbg_mtrace()
#define dbg_trace(level, fmt, args...)
#define dbg_trace_count(level, count, fmt, args...)
#define _dbg_assert(cond, fatal)
#define dbg_assert(cond)
#define dbg_assert_fatal(cond)

#ifdef assert
#undef assert
#endif
#define assert(cond)

#endif

#define dbg_trace_once(level, count, fmt, args...) \
		dbg_trace_count(level, 1, fmt, ## args)

#define dbg_fatal(fmt, args...) \
		dbg_trace(DBG_TRACE_FATAL, fmt, ## args)
#define dbg_fatal_once(fmt, args...) \
		dbg_trace_once(DBG_TRACE_FATAL, fmt, ## args)
#define dbg_error(fmt, args...) \
		dbg_trace(DBG_TRACE_ERROR, fmt, ## args)
#define dbg_error_once(fmt, args...) \
		dbg_trace_once(DBG_TRACE_ERROR, fmt, ## args)
#define dbg_warn(fmt, args...) \
		dbg_trace(DBG_TRACE_WARN, fmt, ## args)
#define dbg_warn_once(fmt, args...) \
		dbg_trace_once(DBG_TRACE_WARN, fmt, ## args)
#define dbg_info(fmt, args...) \
		dbg_trace(DBG_TRACE_INFO, fmt, ## args)
#define dbg_info_once(fmt, args...) \
		dbg_trace_once(DBG_TRACE_INFO, fmt, ## args)
#define dbg_debug(fmt, args...) \
		dbg_trace(DBG_TRACE_DEBUG, fmt, ## args)
#define dbg_debug_once(fmt, args...) \
		dbg_trace_once(DBG_TRACE_DEBUG, fmt, ## args)

#endif /* DBG_H_ */
