#ifndef DDN_DDN_H_
#define DDN_DDN_H_ 1

#include <gmp.h>

#include <stdint.h>
#include <sys/types.h> /* for size_t */
#include <functor.h>
#include <stdio.h>

struct ddn_state_s
{
	union
	{
		uintptr_t num;
		void *ptr;
	} u;
};

typedef struct ddn_state_s ddn_state_t;

struct ddn_node;

struct ddn_state_functor
{
	ddn_state_t (*state_next)(const struct ddn_state_functor *, const struct ddn_node **, size_t);
	int (*func_fprint)(const struct ddn_state_functor *, size_t, FILE *);
};

struct ddn_node
{
	ddn_state_t state;
	ddn_state_t next;
	struct ddn_state_functor *state_next;
	const struct ddn_node **input;
	const struct ddn_node **output;
};

struct ddn
{
	size_t n;
	struct ddn_node *node;
};

static inline ddn_state_t
ddn_state_next(struct ddn_node *node, size_t n)
{
	return node->state_next->state_next(node->state_next, node->input, n);
}

static inline void
ddn_step(struct ddn *ddn)
{
	for (size_t i = 0; i < ddn->n; ++i)
	{
		ddn->node[i].next = ddn_state_next(&ddn->node[i], ddn->n);
	}

	for (size_t i = 0; i < ddn->n; ++i)
	{
		ddn->node[i].state = ddn->node[i].next;
	}
}

int ddn_init(struct ddn *ddn, size_t n);
int ddn_cleanup(struct ddn *ddn);
int ddn_state_sscan(struct ddn *ddn, const char *buf);
int ddn_state_fscan(struct ddn *ddn, FILE *fp);
int ddn_state_fprint(const struct ddn *ddn, FILE *fp);
void ddn_state_toz(const struct ddn *ddn, mpz_t z);
void ddn_state_fromz(struct ddn *ddn, mpz_t z);
int ddn_state_func_fprint(const struct ddn *ddn, size_t n, FILE *fp);

#endif /* DDN_H_ */
