#ifndef DDN_DDN_GT0_H_
#define DDN_DDN_GT0_H_ 1

#include <ga/genotype.h>
#include <util.h>

struct ddn_gt0
{
	struct genotype base;
	size_t count;
	unsigned long bits[];
};

/* Genotype 0 looks like this for n node
 * ilog2z(n)         n
 * +-------+--------------+
 * | param | connections  |
 * +-------+--------------+
 * | param | connections  |
 * +-------+--------------+
 * | ...   | ...          |
 * +-------+--------------+
 * | param | connections  |
 * +-------+--------------+
 *
 * n times.
 *
 * This means the bit count for a genotype for a given network is
 * n (ilog2z(n) + n)
 */

/* Used to calculate how many bits needed for n nodes */
inline static size_t
ddn_gt0_bits(size_t n)
{
	return n * (ilog2z(n) + n);
}

extern const struct genotype_ops ddn_gt0_ops;

struct genotype *ddn_gt0_create(size_t n);
//static int ddn_gt0_destroy(struct genotype *gt);
size_t ddn_gt0_param2num(const unsigned long *bits, size_t offset, size_t len);
void ddn_gt0_num2param(unsigned long *bits, size_t offset, size_t len, size_t num);
int ddn_gt0_sscan(struct genotype *gt, const char *buf);
struct genotype *ddn_gt0_fscan(FILE *fp);
struct phenotype *ddn_gt0_develop(const struct genotype *gt);

#endif
