#ifndef DDN_DDN_PT0_H_
#define DDN_DDN_PT0_H_ 1

#include <dft.h>
#include <ga/phenotype.h>
#include <ddn/ddn.h>
#include <ddn/ddn_gt0.h>

struct ddn_pt0
{
	struct phenotype base;
	struct ddn ddn;
};

struct ddn_pt0_fitness_result
{
	size_t idx;
	double *val;
};

struct ddn_pt0_fitness_data
{
	mpz_t znum;
	mpz_t znum_max;
	mpf_t fnum;
	mpf_t fnum_max;
	struct dft_plan dft;
	double *fitness;
	struct ddn_pt0_fitness_result *sorted_result_max;
	struct ddn_pt0_fitness_result *sorted_result_min;
	mpz_t coverage;
	mpf_t fcoverage;
	mpf_t fcoverage_max;
};

enum ddn_pt0_fitness_operator
{
	DDN_PT0_FITNESS_OPERATOR_GT,
	DDN_PT0_FITNESS_OPERATOR_LT,
	DDN_PT0_FITNESS_OPERATOR_SEP_BLOCK,
};

struct ddn_pt0_fitness_op
{
	size_t op1;
	size_t op2;
	enum ddn_pt0_fitness_operator oper;
	unsigned char padding[4];
};

int ddn_pt0_fitness_init(struct ddn_pt0_fitness_data *fit, size_t window_size,
		double (*window_func)(size_t, size_t), size_t nodes);
int ddn_pt0_fitness_cleanup(struct ddn_pt0_fitness_data *fit);
double ddn_pt0_fitness_sample(struct ddn_pt0_fitness_data *fit, size_t bits);
int ddn_pt0_fitness_test(struct ddn_pt0 *pt0, struct ddn_pt0_fitness_data *fit,
	mpz_t *init_state, size_t init_state_count, size_t state_count,
	const struct ddn_pt0_fitness_op * const op, size_t op_count);
int ddn_pt0_fitness_cleanup(struct ddn_pt0_fitness_data *fit);

struct phenotype *ddn_pt0_create(const struct ddn_gt0 *gt0);
int ddn_pt0_destroy(struct phenotype *pt);
struct ddn *ddn_pt0_ddn(struct phenotype *pt);
size_t ddn_pt0_param(struct phenotype *pt, size_t i);

#endif
