#ifndef DFT_H_
#define DFT_H_ 1

#include <complex.h>
#include <fftw3.h>

#include <sys/types.h>

struct dft_plan
{
	/* fftw3 plan */
	fftw_plan plan;
	/* holds the samples fed into the dft */
	double *sample;
	/* how many samples we have stuffed into the sample array so far */
	size_t sample_count;
	/* how many samples make up a window, is equal to the number of doubles
	 * that fit in the samples array */
	size_t window_size;
	/* how many windows we have transformed so far */
	size_t window_count;
	/* holds the transformed data */
	double complex *transformed;
	/* holds the transformed data */
	size_t transformed_size;
	/* holds the accumulated absolute value of the transformed data */
	double *result;
	/* window function */
	double (*window_func)(size_t, size_t);
};

static inline void
dft_sample_add(struct dft_plan *dft, double sample)
{
	dft->sample[dft->sample_count] = sample * dft->window_func(dft->sample_count,
				dft->window_size);
	//printf("window %f -> %f\n", sample, dft->sample[dft->sample_count]);
	++dft->sample_count;
	if (dft->sample_count == dft->window_size)
	{
		dft->sample_count = 0;
		++dft->window_count;
		fftw_execute(dft->plan);
		for (size_t i = 0; i < dft->transformed_size; ++i)
		{
			dft->result[i] += cabs(dft->transformed[i]);
		}
	}
}


double dft_window_hann(size_t n, size_t total);
double dft_window_hamming(size_t n, size_t total);
double dft_window_rect(size_t n, size_t total);
int dft_plan_init(struct dft_plan *plan, size_t window_size,
		double (*window_func)(size_t, size_t));
void dft_init(struct dft_plan *dft);
void dft_finish(struct dft_plan *dft);
void dft_plan_cleanup(struct dft_plan *dft);
double (*dft_window_func_from_str(const char *func))(size_t, size_t);

#endif /* DFT_H_ */
