#ifndef GA_H_
#define GA_H_ 1

#include <dtable.h>

#include <ga/genotype.h>
#include <ga/phenotype.h>
#include <ga/individual.h>

#include <stdio.h>

#define GA_DTABLE_SIZE 256

struct ga;

struct ga_ops
{
	/* create genotypes */
	struct genotype *(*genotype_create)(void);
	/* create phenotypes */
	struct phenotype *(*phenotype_create)(const struct genotype *);
	/* test fitness of phenotype */
	int (*phenotype_test_fitness)(struct phenotype *, const struct dtable *);
	/* adult selection */
	int (*adult_sel_proto)(struct ga *);
	int (*adult_sel_mech)(struct ga *);
	/* parent selection */
	//int (*parent_sel_proto)(struct ga *);
	int (*parent_sel_mech)(struct ga *);
	/* FIXME: all items below are not fixed yet */
	//int (*adult_selection)(struct ga *);
	//int (*parent_selection)(struct ga *);
	//void (*statistics_print)(struct ga *, FILE *);
};

struct ga_settings
{
	size_t adults_count;
	size_t children_count;
	size_t crossover_points;
	double crossover_prob;
	size_t elites_count;
	size_t generation_print;
	size_t generation_count;
	double mutation_prob;
	FILE *population_file;
	FILE *fitness_file;
};

struct ga
{
	/* table of all adults in current generation */
	struct dtable adults;
	/* table of all children in current generation */
	struct dtable children;
	/* table of all parents in current generation */
	struct dtable parents;
	/* operations to perform on GA */
	struct ga_ops *ops;
	/* GA settings */
	struct ga_settings *settings;
	/* number of mutation possibilities there has been */
	size_t mutation_possi;
	/* number of acual mutations done */
	size_t mutation_count;
	/* number of crossover possibilities there has been */
	size_t crossover_possi;
	/* number of actual crossovers done */
	size_t crossover_count;
	/* standards deviation of current generations fitness */
	double fitness_stddev;
	/* average of currect generations fitness */
	double fitness_avg;
	/* best fitness in current generation */
	double fitness_best;
	/* best individual in current generation */
	struct individual *best;
	/* current generation count */
	size_t generation;
	struct prand *prand;
};

int ga_init(struct ga *ga, struct ga_ops *ops, struct ga_settings *settings);
int ga_children_develop(struct ga *ga);
int ga_cleanup(struct ga *ga);
int ga_children_test_fitness(struct ga *ga);
int ga_adult_selection(struct ga *ga);
int ga_parent_selection(struct ga *ga);
int ga_reproduce(struct ga *ga);
int ga_run(struct ga *ga);

int fitness_proportionate_selection(struct ga *ga);
int sigma_scale_selection(struct ga *ga);
int boltzman_scale(struct ga *ga, double temp);
int tournament_selection(struct ga *ga, size_t k, double e);


int ga_adult_sel_proto_full_gen_rep(struct ga *ga);
int ga_adult_sel_proto_gen_mix(struct ga *ga);
int ga_adult_sel_mech_best_fitness(struct ga *ga);
int ga_adult_sel_mech_stochastic(struct ga *ga);

#endif
