#ifndef GENOTYPE_H_
#define GENOTYPE_H_ 1

#include <stdio.h>
#include <stddef.h>

struct genotype;
struct prand;

struct genotype_ops
{
	int (*mutate)(struct genotype *, struct prand *, size_t);
	int (*crossover)( struct genotype *, struct genotype *, size_t, struct prand *);
	int (*init_random)(struct genotype *, struct prand *);
	void (*fprint)(const struct genotype *, FILE *out);
	struct genotype *(*clone)(const struct genotype *);
};

struct genotype
{
	const struct genotype_ops *ops;
	int (*cleanup)(struct genotype *);
	size_t features;
};

static inline int
genotype_cleanup(struct genotype *gt)
{
	return gt->cleanup(gt);
}

static inline int
genotype_mutate(struct genotype *gt, struct prand *prand, size_t feature)
{
	return gt->ops->mutate(gt, prand, feature);
}

static inline int
genotype_crossover(struct genotype *child1, struct genotype *child2, size_t n, struct prand *prand)
{
	return child1->ops->crossover(child1, child2, n, prand);
}

static inline int
genotype_init_random(struct genotype *gt, struct prand *prand)
{
	return gt->ops->init_random(gt, prand);
}

static inline void
genotype_fprint(const struct genotype *gt, FILE *out)
{
	return gt->ops->fprint(gt, out);
}

static inline struct genotype *
genotype_clone(const struct genotype *gt)
{
	return gt->ops->clone(gt);
}

#endif
