#ifndef UTIL_H_
#define UTIL_H_ 1

#include <dbg.h>
#include <stddef.h>

#ifndef structof
#define structof(ptr, member, type) ((type *)((char *)(ptr) - ((size_t)&((type *)0)->member)))
#endif

//		typeof(a) __a = a;
#define ARRAY_SIZE(a) \
	({ \
		sizeof(a) / sizeof(a[0]); \
	})

#define stringify1(a) #a
#define stringify(a) stringify1(a)

#define barrier() \
	({ \
		asm volatile ("" ::: "memory"); \
	})

#define std_create(type, ...) \
	({\
		struct type *__p; \
		__p = malloc(sizeof(*__p)); \
		if (!__p) \
			dbg_fatal(#type "_create(): %s", \
				strerror(errno)); \
		else \
		{ \
			int __ret; \
			__ret = type ## _init(__p, ##__VA_ARGS__); \
			if (__ret) \
			{ \
				dbg_error(#type "_create(): %s", \
					strerror(-__ret)); \
				errno = -__ret; \
				free(__p); \
				__p = NULL; \
			} \
		} \
		__p; \
	})

#define std_destroy(type, ptr, ...) \
	({ \
		int __ret; \
		__ret = type ## _cleanup(ptr, ##_VA_ARGS__); \
		if (__ret) \
		{ \
			dbg_error(#type "_destroy(): %s", \
				strerror(-__ret)); \
		} \
		free(ptr); \
		__ret; \
	})

#define BITS_ULONG (sizeof(unsigned long) * 8UL)
#define bit_count(x) (((x) + (BITS_ULONG - 1UL)) / BITS_ULONG)
#define bit_isset(array, bit) (((((array)[(bit) / BITS_ULONG]) & (1UL << ((bit) % BITS_ULONG))) >> ((bit) % BITS_ULONG)) & 1UL)
#define BIT_DEFINE(array, bits) (unsigned long (array)[(bits)])
#define bit_get(array, bit) ((array)[(bit) / BITS_ULONG])
#define bit_set(array, bit) (bit_get(array, bit) |= (1UL << ((bit) % BITS_ULONG)))
#define bit_clr(array, bit) (bit_get(array, bit) &= ~(1UL << ((bit) % BITS_ULONG)))
#define bit_flip(array, bit) (bit_get(array, bit) ^= (1UL << ((bit) % BITS_ULONG)))

#define MAX(a, b) \
	({ \
		typeof(a) __a = a; \
		typeof(b) __b = b; \
		__a > __b ? __a : __b; \
	})

#define MIN(a, b) \
	({ \
		typeof(a) __a = a; \
		typeof(b) __b = b; \
		__a < __b ? __a : __b; \
	})

inline static size_t
ilog2z(size_t n)
{
	size_t val, i;
	val = n;

	i = 0;
	while (val >>= 1)
		++i;

	return i;
}

#endif /* UTIL_H_ */
