#include <dbg.h>
#include <ddn/ddn.h>

#include <gmp.h>

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

int
ddn_init(struct ddn *ddn, size_t n)
{
	struct ddn_node *base;
	int ret;

	ret = 0;
	base = calloc(n, sizeof(*(ddn->node)));
	if (!base)
	{
		ret = -errno;
		dbg_error("%s\n", strerror(-ret));
		goto base_alloc_fail;
	}

	for (size_t i = 0; i < n; ++i)
	{
		const struct ddn_node **input;
		const struct ddn_node **output;

		input = calloc(n, sizeof(ddn->node));
		if (!input)
		{
			ret = -errno;
			dbg_error("%s\n", strerror(-ret));
			goto conn_alloc_fail;
		}

		output = calloc(n, sizeof(ddn->node));
		if (!output)
		{
			ret = -errno;
			dbg_error("%s\n", strerror(-ret));
			free(input);
			goto conn_alloc_fail;
		}

		base[i].input = input;
		base[i].output = output;
		base[i].state_next = NULL;
	}
	ddn->n = n;
	ddn->node = base;

	return ret;

conn_alloc_fail:
	for (size_t i = 0; i < n; ++i)
	{
		if (base[i].input)
			free(base[i].input);
		if (base[i].output)
			free(base[i].output);
	}
base_alloc_fail:
	return ret;
}

int
ddn_cleanup(struct ddn *ddn)
{
	for (size_t i = 0; i < ddn->n; ++i)
	{
		//if (ddn->node[i].state_next)
		//	free(ddn->node[i].state_next);
		free(ddn->node[i].input);
		free(ddn->node[i].output);
	}
	free(ddn->node);
	ddn->node = NULL;
	ddn->n = 0;
	return 0;
}

/* FIXME: only supports boolean */
int
ddn_state_fscan(struct ddn *ddn, FILE *fp)
{
	int ret, c;
	size_t count;

	ret = 0;
	do
	{
		c = fgetc(fp);
		if (c == EOF)
		{
			if (!feof(fp))
			{
				ret = -errno;
				dbg_error("%s\n", strerror(errno));
			}
			else
				ret = 1;
			goto err_out;
		}
	} while (isspace(c));

	for (count = 0; count < ddn->n && !isspace(c) && c != EOF; ++count, c = fgetc(fp))
	{
		if (c == '0' || c == '1')
		{
			ddn->node[ddn->n - (count + 1)].state.u.num = c - '0';
		}
		else
		{
			dbg_error("invalid character '%c'\n", c);
			ret = -EILSEQ;
			goto err_out;
		}
	}

	if (count != ddn->n)
	{
		dbg_error("got %zu bits, expected %zu\n", count, ddn->n);
		ret = -EILSEQ;
		goto err_out;
	}
	else if (c != EOF && !isspace(c))
	{
		dbg_error("garbage at end of input\n");
		ret = -EILSEQ;
		goto err_out;
	}
	else if (c == EOF && !feof(fp))
	{
		int tmp = errno;
		dbg_error("%s\n", strerror(errno));
		errno = tmp;
		goto err_out;
	}

err_out:
	return ret;
}

int
ddn_state_fprint(const struct ddn *ddn, FILE *fp)
{
	for (size_t i = ddn->n; i > 0; --i)
	{
		fputc(ddn->node[i - 1].state.u.num + '0', fp);
	}
	//fputc('\n', fp);
	return 0;
}

void
ddn_state_toz(const struct ddn *ddn, mpz_t z)
{
	mpz_set_ui(z, 0);
	for (size_t i = ddn->n; i > 0; --i)
	{
		mpz_mul_2exp(z, z, 1);
		if (ddn->node[i - 1].state.u.num)
			mpz_setbit(z, 0);
	}
}

void
ddn_state_fromz(struct ddn *ddn, mpz_t z)
{
	for (size_t i = 0; i < ddn->n; ++i)
	{
		ddn->node[i].state.u.num = mpz_tstbit(z, i);
	}
}

int
ddn_state_func_fprint(const struct ddn *ddn, size_t i, FILE *fp)
{
	return ddn->node[i].state_next->func_fprint(ddn->node[i].state_next, ddn->n, fp);
}
