#include <uprand.h>
#include <prand.h>
#include <util.h>

#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

static int
uprand(struct prand *pr)
{
	struct uprand *upr;

	upr = structof(pr, base, struct uprand);
	return rand_r(&upr->seed);
}

static const struct prand_ops
uprand_ops =
{
	.prand = uprand,
};

static int
uprand_cleanup(struct prand *pr)
{
	return 0;
}

static int
uprand_destroy(struct prand *pr)
{
	struct uprand *upr;
	int ret;

	upr = structof(pr, base, struct uprand);
	ret = uprand_cleanup(pr);
	free(upr);
	return ret;
}

struct prand *
uprand_create(void)
{
	struct uprand *upr;
	int ret;

	upr = malloc(sizeof(*upr));
	if (!upr)
	{
		ret = errno;
		dbg_error("%s\n", strerror(ret));
		errno = ret;
		goto err;
	}

	ret = uprand_init(upr);
	if (ret)
	{
		dbg_error("%s\n", strerror(-ret));
		free(upr);
		errno = -ret;
		goto err;
	}
	upr->base.cleanup = uprand_destroy;

	return &upr->base;
err:
	return NULL;
}

int
uprand_init(struct uprand *upr)
{
	int fd, ret;
	unsigned int seed;

	ret = 0;
	fd = open("/dev/urandom", 0);
	if (fd == -1)
	{
		ret = -errno;
		dbg_error("%s\n", strerror(-ret));
		return ret;
	}

	if (read(fd, &seed, sizeof(seed)) != sizeof(seed))
	{
		ret = -errno;
		dbg_error("%s\n", strerror(-ret));
		goto out;
	}

	upr->base.ops = &uprand_ops;
	upr->seed = seed;
	upr->base.cleanup = uprand_cleanup;
out:
	if (close(fd))
	{
		ret = -errno;
		dbg_error("%s\n", strerror(-ret));
	}

	return ret;
}
